@@ -1,6 +1,6 @@
 /****************************************************************************
  *
- *  $Id: linphonec.c,v 1.52 2007/01/10 14:11:24 smorlat Exp $
+ *  $Id: linphonec.c,v 1.53 2007/02/13 21:31:01 smorlat Exp $
  *
  *  Copyright (C) 2006  Sandro Santilli <strk@keybit.net>
  *  Copyright (C) 2002  Florian Winterstein <flox@gmx.net>
@@ -131,7 +131,8 @@ LinphoneCoreVTable linphonec_vtable = {
 	display_warning:linphonec_display_warning,
 	display_url:linphonec_display_url,
 	display_question:stub,
-	text_received:linphonec_text_received
+	text_received:linphonec_text_received,
+        general_state:NULL
 };
 
 /***************************************************************************
@@ -928,6 +929,11 @@ lpc_strip_blanks(char *input)
 /****************************************************************************
  *
  * $Log: linphonec.c,v $
+ * Revision 1.53  2007/02/13 21:31:01  smorlat
+ * added patch for general state.
+ * new doxygen for oRTP
+ * gtk-doc removed.
+ *
  * Revision 1.52  2007/01/10 14:11:24  smorlat
  * add --video to linphonec.
  *
