@@ -104,14 +104,18 @@ LinphoneCall * linphone_call_new_incoming(LinphoneCore *lc, const char *from, co
 	char localip[LINPHONE_IPADDR_SIZE];
 	LinphoneCall *call=ms_new0(LinphoneCall,1);
 	osip_from_t *me= linphone_core_get_primary_contact_parsed(lc);
+	osip_from_t *from_url=NULL;
 	call->dir=LinphoneCallIncoming;
 	call->cid=cid;
 	call->did=did;
 	call->core=lc;
-	linphone_core_get_local_ip(lc,NULL,localip);
+	osip_from_init(&from_url);
+	osip_from_parse(from_url, from);
+	linphone_core_get_local_ip(lc,from_url->url->host,localip);
 	call->sdpctx=sdp_handler_create_context(&linphone_sdphandler,localip,me->url->username);
 	linphone_call_init_common(call, osip_strdup(from), osip_strdup(to));
 	osip_from_free(me);
+	osip_from_free(from_url);
 	return call;
 }
 
@@ -131,7 +135,6 @@ static size_t my_strftime(char *s, size_t max, const char  *fmt,  const struct t
 LinphoneCallLog * linphone_call_log_new(LinphoneCall *call, char *from, char *to){
 	LinphoneCallLog *cl=ms_new0(LinphoneCallLog,1);
 	struct tm loctime;
-	cl->lc=call->core;
 	cl->dir=call->dir;
 #ifdef WIN32
 	loctime=*localtime(&call->start_time);
@@ -528,6 +531,14 @@ const char * linphone_core_get_version(void){
 void
 linphone_core_init (LinphoneCore * lc, const LinphoneCoreVTable *vtable, const char *config_path, void * userdata)
 {
+	memset (lc, 0, sizeof (LinphoneCore));
+	lc->data=userdata;
+	
+	memcpy(&lc->vtable,vtable,sizeof(LinphoneCoreVTable));
+
+        gstate_initialize();
+        gstate_new_state(lc, GSTATE_POWER_STARTUP, NULL);
+        
 	ortp_init();
 	rtp_profile_set_payload(&av_profile,115,&payload_type_lpc1015);
 	rtp_profile_set_payload(&av_profile,110,&payload_type_speex_nb);
@@ -542,11 +553,6 @@ linphone_core_init (LinphoneCore * lc, const LinphoneCoreVTable *vtable, const c
 
 	ms_init();
 	
-	memset (lc, 0, sizeof (LinphoneCore));
-	lc->data=userdata;
-	
-	memcpy(&lc->vtable,vtable,sizeof(LinphoneCoreVTable));
-
 	lc->config=lp_config_new(config_path);
 
   
@@ -568,6 +574,7 @@ linphone_core_init (LinphoneCore * lc, const LinphoneCoreVTable *vtable, const c
 	ui_config_read(lc);
 	ms_mutex_init(&lc->lock,NULL);
 	lc->vtable.display_status(lc,_("Ready"));
+        gstate_new_state(lc, GSTATE_POWER_ON, NULL);
 }
 
 LinphoneCore *linphone_core_new(const LinphoneCoreVTable *vtable,
@@ -633,7 +640,7 @@ void linphone_core_get_local_ip(LinphoneCore *lc, const char *dest, char *result
 	char *tmp=NULL;
 	if (lc->apply_nat_settings){
 		apply_nat_settings(lc);
-		lc->apply_nat_settings=false;
+		lc->apply_nat_settings=FALSE;
 	}
 	if (linphone_core_get_firewall_policy(lc)==LINPHONE_POLICY_USE_NAT_ADDRESS){
 		strncpy(result,linphone_core_get_nat_address(lc),LINPHONE_IPADDR_SIZE);
@@ -1013,9 +1020,11 @@ int linphone_core_invite(LinphoneCore *lc, const char *url)
 		lc->vtable.display_warning(lc,_("Sorry, having multiple simultaneous calls is not supported yet !"));
 	}
 
+	gstate_new_state(lc, GSTATE_CALL_OUT_INVITE, url);
 	linphone_core_get_default_proxy(lc,&proxy);
 	if (!linphone_core_interpret_url(lc,url,&real_url,&real_parsed_url)){
 		/* bad url */
+		gstate_new_state(lc, GSTATE_CALL_ERROR, NULL);
 		return -1;
 	}
 	barmsg=ortp_strdup_printf("%s %s", _("Contacting"), real_url);
@@ -1044,6 +1053,7 @@ int linphone_core_invite(LinphoneCore *lc, const char *url)
 	ctx=lc->call->sdpctx;
 	sdpmesg=sdp_context_get_offer(ctx);
 	eXosip_lock();
+	linphone_core_init_media_streams(lc);
 	err=eXosip_initiate_call_with_body(invite,"application/sdp",sdpmesg,(void*)lc->call);
 	lc->call->cid=err;
 	eXosip_unlock();
@@ -1052,6 +1062,7 @@ int linphone_core_invite(LinphoneCore *lc, const char *url)
 		lc->vtable.display_status(lc,_("could not call"));
 		linphone_call_destroy(lc->call);
 		lc->call=NULL;
+		linphone_core_stop_media_streams(lc);
 	}
 	
 	goto end;
@@ -1059,6 +1070,8 @@ int linphone_core_invite(LinphoneCore *lc, const char *url)
 		if (real_url!=NULL) ms_free(real_url);
 		if (real_parsed_url!=NULL) osip_to_free(real_parsed_url);
 		if (parsed_url2!=NULL) osip_from_free(parsed_url2);
+		if (err<0)
+			gstate_new_state(lc, GSTATE_CALL_ERROR, NULL);
 	return (err<0) ? -1 : 0;
 }
 
@@ -1164,12 +1177,18 @@ int linphone_core_change_qos(LinphoneCore *lc, int answer)
 }
 #endif
 
+void linphone_core_init_media_streams(LinphoneCore *lc){
+	lc->audiostream=audio_stream_new(linphone_core_get_audio_port(lc),linphone_core_ipv6_enabled(lc));
+	lc->videostream=video_stream_new(linphone_core_get_video_port(lc),linphone_core_ipv6_enabled(lc));
+}
+
 void linphone_core_start_media_streams(LinphoneCore *lc, LinphoneCall *call){
 	osip_from_t *me=linphone_core_get_primary_contact_parsed(lc);
 	const char *tool="linphone-" LINPHONE_VERSION;
+	/* adjust rtp jitter compensation. It must be at least the latency of the sound card */
+	int jitt_comp=MAX(lc->sound_conf.latency,lc->rtp_conf.audio_jitt_comp);
 	char *cname=ortp_strdup_printf("%s@%s",me->url->username,me->url->host);
 	{
-		int jitt_comp;
 		StreamParams *audio_params=&call->audio_params;
 		if (!lc->use_files){
 			MSSndCard *playcard=lc->sound_conf.play_sndcard;
@@ -1183,11 +1202,9 @@ void linphone_core_start_media_streams(LinphoneCore *lc, LinphoneCall *call){
 				goto end;
 			}
 			
-			/* adjust rtp jitter compensation. It must be at least the latency of the sound card */
-			jitt_comp=MAX(lc->sound_conf.latency,lc->rtp_conf.audio_jitt_comp);
-			lc->audiostream=audio_stream_start_with_sndcards(
+			audio_stream_start_now(
+				lc->audiostream,
 				call->profile,
-				audio_params->localport,
 				audio_params->remoteaddr,
 				audio_params->remoteport,
 				audio_params->pt,
@@ -1196,9 +1213,9 @@ void linphone_core_start_media_streams(LinphoneCore *lc, LinphoneCall *call){
 				captcard,
 				linphone_core_echo_cancelation_enabled(lc));
 		}else{
-			lc->audiostream=audio_stream_start_with_files(
+			audio_stream_start_with_files(
+				lc->audiostream,
 				call->profile,
-				audio_params->localport,
 				audio_params->remoteaddr,
 				audio_params->remoteport,
 				audio_params->pt,
@@ -1206,12 +1223,7 @@ void linphone_core_start_media_streams(LinphoneCore *lc, LinphoneCall *call){
 				lc->play_file,
 				lc->rec_file);
 		}
-		if (lc->audiostream==NULL){
-			/* should terminate call */
-		}
-		else{
-			audio_stream_set_rtcp_information(lc->audiostream, cname, tool);
-		}
+		audio_stream_set_rtcp_information(lc->audiostream, cname, tool);
 	}
 #ifdef VIDEO_ENABLED
 	{
@@ -1221,15 +1233,13 @@ void linphone_core_start_media_streams(LinphoneCore *lc, LinphoneCall *call){
 			lc->previewstream=NULL;
 		}
 		if (lc->video_conf.enabled){
-			int jitt_comp;
 			StreamParams *video_params=&call->video_params;
 			
 			if (video_params->remoteport>0){
-				/* adjust rtp jitter compensation. It must be at least the latency of the sound card */
-				jitt_comp=MAX(lc->sound_conf.latency,lc->rtp_conf.audio_jitt_comp);
-				lc->videostream=video_stream_start(call->profile,video_params->localport,
-									video_params->remoteaddr,video_params->remoteport,
-									video_params->pt,jitt_comp, lc->video_conf.device);
+				
+				video_stream_start(lc->videostream, call->profile,
+				video_params->remoteaddr,video_params->remoteport,
+					video_params->pt,jitt_comp, lc->video_conf.device);
 				video_stream_set_rtcp_information(lc->videostream, cname,tool);
 			}
 		}
@@ -1284,6 +1294,7 @@ int linphone_core_accept_call(LinphoneCore *lc, const char *url)
 	eXosip_answer_call_with_body(lc->call->did,200,"application/sdp",sdpmesg);
 	eXosip_unlock();
 	lc->vtable.display_status(lc,_("Connected."));
+	gstate_new_state(lc, GSTATE_CALL_IN_CONNECTED, NULL);
 	
 	linphone_core_start_media_streams(lc, lc->call);
 	return 0;
@@ -1308,6 +1319,7 @@ int linphone_core_terminate_call(LinphoneCore *lc, const char *url)
 	}
 	linphone_core_stop_media_streams(lc);
 	lc->vtable.display_status(lc,_("Call ended") );
+	gstate_new_state(lc, GSTATE_CALL_END, NULL);
 	linphone_call_destroy(call);
 	return 0;
 }
@@ -1451,6 +1463,7 @@ void linphone_core_set_sound_source(LinphoneCore *lc, char source)
 {
 	MSSndCard *sndcard=lc->sound_conf.capt_sndcard;
 	lc->sound_conf.source=source;
+	if (!sndcard) return;
 	switch(source){
 		case 'm':
 			ms_snd_card_set_capture(sndcard,MS_SND_CARD_MIC);
@@ -1557,7 +1570,11 @@ static void apply_nat_settings(LinphoneCore *lc){
 	struct addrinfo hints,*res;
 	const char *addr=lc->net_conf.nat_address;
 	
-	if (lc->net_conf.firewall_policy==LINPHONE_POLICY_USE_NAT_ADDRESS && addr!=NULL && strlen(addr)>0){
+	if (lc->net_conf.firewall_policy==LINPHONE_POLICY_USE_NAT_ADDRESS){
+		if (addr==NULL || strlen(addr)==0){
+			lc->vtable.display_warning(lc,_("No nat/firewall address supplied !"));
+			linphone_core_set_firewall_policy(lc,LINPHONE_POLICY_NO_FIREWALL);
+		}
 		/*check the ip address given */
 		memset(&hints,0,sizeof(struct addrinfo));
 		if (lc->sip_conf.ipv6_enabled)
@@ -1859,6 +1876,7 @@ LpConfig *linphone_core_get_config(LinphoneCore *lc){
 
 void linphone_core_uninit(LinphoneCore *lc)
 {
+	gstate_new_state(lc, GSTATE_POWER_SHUTDOWN, NULL);
 #ifdef VIDEO_ENABLED
 	if (lc->previewstream!=NULL){
 		video_preview_stop(lc->previewstream);
@@ -1880,6 +1898,7 @@ void linphone_core_uninit(LinphoneCore *lc)
 	ortp_exit();
 	eXosip_quit();
 	exosip_running=FALSE;
+	gstate_new_state(lc, GSTATE_POWER_OFF, NULL);
 }
 
 void linphone_core_destroy(LinphoneCore *lc){
