@@ -24,18 +24,15 @@
 
 #define MAX_LEN 1024
 
-#include <config.h>
-
-#ifdef HAVE_GLIB
-#include <glib.h>
-#else
-#include <uglib.h>
-#endif
+#include "linphonecore.h"
 
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
 #include <assert.h>
+#include <errno.h>
+#include <sys/types.h>
+#include <sys/stat.h>
 
 #define lp_new0(type,n)	(type*)calloc(sizeof(type),n)
 
@@ -92,13 +89,13 @@ typedef struct _LpItem{
 
 typedef struct _LpSection{
 	char *name;
-	GList *items;
+	MSList *items;
 } LpSection;
 
 struct _LpConfig{
 	FILE *file;
 	char *filename;
-	GList *sections;
+	MSList *sections;
 };
 
 LpItem * lp_item_new(const char *key, const char *value){
@@ -114,7 +111,8 @@ LpSection *lp_section_new(const char *name){
 	return sec;
 }
 
-void lp_item_destroy(LpItem *item){
+void lp_item_destroy(void *pitem){
+	LpItem *item=(LpItem*)pitem;
 	free(item->key);
 	free(item->value);
 	free(item);
@@ -122,25 +120,25 @@ void lp_item_destroy(LpItem *item){
 
 void lp_section_destroy(LpSection *sec){
 	free(sec->name);
-	g_list_foreach(sec->items,(GFunc)lp_item_destroy,NULL);
-	g_list_free(sec->items);
+	ms_list_for_each(sec->items,lp_item_destroy);
+	ms_list_free(sec->items);
 	free(sec);
 }
 
 void lp_section_add_item(LpSection *sec,LpItem *item){
-	sec->items=g_list_append(sec->items,(gpointer)item);
+	sec->items=ms_list_append(sec->items,(void *)item);
 }
 
 void lp_config_add_section(LpConfig *lpconfig, LpSection *section){
-	lpconfig->sections=g_list_append(lpconfig->sections,(gpointer)section);
+	lpconfig->sections=ms_list_append(lpconfig->sections,(void *)section);
 }
 
 void lp_config_remove_section(LpConfig *lpconfig, LpSection *section){
-	lpconfig->sections=g_list_remove(lpconfig->sections,(gpointer)section);
+	lpconfig->sections=ms_list_remove(lpconfig->sections,(void *)section);
 	lp_section_destroy(section);
 }
 
-static gboolean is_first_char(const char *start, const char *pos){
+static bool_t is_first_char(const char *start, const char *pos){
 	const char *p;
 	for(p=start;p<pos;p++){
 		if (*p!=' ') return FALSE;
@@ -172,7 +170,7 @@ void lp_config_parse(LpConfig *lpconfig){
 						lp_config_add_section(lpconfig,cur);
 					}
 				}else{
-					g_warning("parse error!");
+					ms_warning("parse error!");
 				}
 			}
 		}else {
@@ -199,7 +197,7 @@ void lp_config_parse(LpConfig *lpconfig){
 							lp_section_add_item(cur,lp_item_new(key,pos1));
 							/*printf("Found %s %s=%s\n",cur->name,key,pos1);*/
 						}else{
-							g_warning("found key,item but no sections");
+							ms_warning("found key,item but no sections");
 						}
 					}
 				}
@@ -216,6 +214,11 @@ LpConfig * lp_config_new(const char *filename){
 		if (lpconfig->file!=NULL){
 			lp_config_parse(lpconfig);
 			fclose(lpconfig->file);
+			/* make existing configuration files non-group/world-accessible */
+			if (chmod(filename, S_IRUSR | S_IWUSR) == -1)
+				ms_warning("unable to correct permissions on "
+				  	  "configuration file: %s",
+					   strerror(errno));
 			lpconfig->file=NULL;
 		}
 	}
@@ -230,16 +233,16 @@ void lp_item_set_value(LpItem *item, const char *value){
 
 void lp_config_destroy(LpConfig *lpconfig){
 	if (lpconfig->filename!=NULL) free(lpconfig->filename);
-	g_list_foreach(lpconfig->sections,(GFunc)lp_section_destroy,NULL);
-	g_list_free(lpconfig->sections);
+	ms_list_for_each(lpconfig->sections,(void (*)(void*))lp_section_destroy);
+	ms_list_free(lpconfig->sections);
 	free(lpconfig);
 }
 
 LpSection *lp_config_find_section(LpConfig *lpconfig, const char *name){
 	LpSection *sec;
-	GList *elem;
+	MSList *elem;
 	/*printf("Looking for section %s\n",name);*/
-	for (elem=lpconfig->sections;elem!=NULL;elem=g_list_next(elem)){
+	for (elem=lpconfig->sections;elem!=NULL;elem=ms_list_next(elem)){
 		sec=(LpSection*)elem->data;
 		if (strcmp(sec->name,name)==0){
 			/*printf("Section %s found\n",name);*/
@@ -250,10 +253,10 @@ LpSection *lp_config_find_section(LpConfig *lpconfig, const char *name){
 }
 
 LpItem *lp_section_find_item(LpSection *sec, const char *name){
-	GList *elem;
+	MSList *elem;
 	LpItem *item;
 	/*printf("Looking for item %s\n",name);*/
-	for (elem=sec->items;elem!=NULL;elem=g_list_next(elem)){
+	for (elem=sec->items;elem!=NULL;elem=ms_list_next(elem)){
 		item=(LpItem*)elem->data;
 		if (strcmp(item->key,name)==0) {
 			/*printf("Item %s found\n",name);*/
@@ -264,7 +267,7 @@ LpItem *lp_section_find_item(LpSection *sec, const char *name){
 }
 
 void lp_section_remove_item(LpSection *sec, LpItem *item){
-	sec->items=g_list_remove(sec->items,(gpointer)item);
+	sec->items=ms_list_remove(sec->items,(void *)item);
 	lp_item_destroy(item);
 }
 
@@ -317,19 +320,23 @@ void lp_item_write(LpItem *item, FILE *file){
 
 void lp_section_write(LpSection *sec, FILE *file){
 	fprintf(file,"[%s]\n",sec->name);
-	g_list_foreach(sec->items,(GFunc)lp_item_write,(gpointer)file);
+	ms_list_for_each2(sec->items,(void (*)(void*, void*))lp_item_write,(void *)file);
 	fprintf(file,"\n");
 }
 
 int lp_config_sync(LpConfig *lpconfig){
 	FILE *file;
 	if (lpconfig->filename==NULL) return -1;
+#ifndef WIN32
+	/* don't create group/world-accessible files */
+	(void) umask(S_IRWXG | S_IRWXO);
+#endif
 	file=fopen(lpconfig->filename,"w");
 	if (file==NULL){
-		g_warning("Could not write %s !",lpconfig->filename);
+		ms_warning("Could not write %s !",lpconfig->filename);
 		return -1;
 	}
-	g_list_foreach(lpconfig->sections,(GFunc)lp_section_write,(gpointer)file);
+	ms_list_for_each2(lpconfig->sections,(void (*)(void *,void*))lp_section_write,(void *)file);
 	fclose(file);
 	return 0;
 }
