@@ -459,8 +459,9 @@ int   eXosip_answer_options(int cid, int jid, int status);
  * @param from      SIP url for caller.
  * @param proxy     Proxy used for registration.
  * @param contact   Contact address. (optionnal)
+ * @param route     Outbound proxy address. (optionnal)
  */
-int   eXosip_register_init (char *from, char *proxy, char *contact);
+int   eXosip_register_init (char *from, char *proxy, char *contact, char* route);
 
 /**
  * Update the registration. (REGISTER request)
@@ -591,6 +592,8 @@ int  eXosip_notify(int nid, int subscription_status, int online_status);
 /** @} */
 
 
+int eXosip_send_ack(int did);
+
 /**
  * @defgroup eXosip_event eXosip event Management
  * @ingroup eXosip
