@@ -64,7 +64,10 @@ typedef enum MSFilterId{
 	MS_PASND_WRITE_ID,
 	MS_MPEG4_ENC_ID,
 	MS_MPEG4_DEC_ID,
-	MS_MJPEG_DEC_ID
+	MS_MJPEG_DEC_ID,
+	MS_JOIN_ID,
+	MS_RESAMPLE_ID,
+	MS_VIDEO_OUT_ID
 } MSFilterId;
 
 
