@@ -42,6 +42,9 @@ struct _MSTicker
 	MSTickerTimeFunc get_cur_time_ptr;
 	void *get_cur_time_data;
 	bool_t run;       /* flag to indicate whether the ticker must be run or not */
+#ifdef WIN32_TIMERS
+	HANDLE TimeEvent;
+#endif
 };
 
 typedef struct _MSTicker MSTicker;
