@@ -145,6 +145,9 @@ static snd_pcm_t * alsa_open_r(const char *pcmdev,int bits,int stereo,int rate)
 	snd_pcm_t *pcm_handle;
 	int err;
 
+	ms_message("alsa_open_r: opening %s at %iHz, bits=%i, stereo=%i",pcmdev,rate,bits,stereo);
+
+
 	if (snd_pcm_open(&pcm_handle, pcmdev,SND_PCM_STREAM_CAPTURE,SND_PCM_NONBLOCK) < 0) {
 		ms_warning("alsa_card_open_r: Error opening PCM device %s\n",pcmdev );
 		return NULL;
@@ -155,7 +158,7 @@ static snd_pcm_t * alsa_open_r(const char *pcmdev,int bits,int stereo,int rate)
 	}
 	err=snd_pcm_start(pcm_handle);
 	if (err<0){
-		ms_warning("Cannot start read pcm: %s", snd_strerror(err));
+		ms_warning("snd_pcm_start() failed: %s", snd_strerror(err));
 	}
 	return pcm_handle;
 }
@@ -250,9 +253,9 @@ static int get_mixer_element(snd_mixer_t *mixer,const char *name, MixerAction ac
 	const char *elemname;
 	snd_mixer_elem_t *elem;
 	int err;
-	long sndMixerPMin;
-	long sndMixerPMax;
-	long newvol;
+	long sndMixerPMin=0;
+	long sndMixerPMax=0;
+	long newvol=0;
 	elem=snd_mixer_first_elem(mixer);
 	while (elem!=NULL){
 		elemname=snd_mixer_selem_get_name(elem);
@@ -261,7 +264,7 @@ static int get_mixer_element(snd_mixer_t *mixer,const char *name, MixerAction ac
 			switch (action){
 				case CAPTURE:
 				if (snd_mixer_selem_has_capture_volume(elem)){
-					snd_mixer_selem_get_playback_volume_range(elem, &sndMixerPMin, &sndMixerPMax);
+					snd_mixer_selem_get_capture_volume_range(elem, &sndMixerPMin, &sndMixerPMax);
 					err=snd_mixer_selem_get_capture_volume(elem,SND_MIXER_SCHN_UNKNOWN,&newvol);
 					newvol-=sndMixerPMin;
 					value=(100*newvol)/(sndMixerPMax-sndMixerPMin);
@@ -299,9 +302,9 @@ static int get_mixer_element(snd_mixer_t *mixer,const char *name, MixerAction ac
 static void set_mixer_element(snd_mixer_t *mixer,const char *name, int level,MixerAction action){
 	const char *elemname;
 	snd_mixer_elem_t *elem;
-	long sndMixerPMin;
-	long sndMixerPMax;
-	long newvol;
+	long sndMixerPMin=0;
+	long sndMixerPMax=0;
+	long newvol=0;
 	
 	elem=snd_mixer_first_elem(mixer);
 	
@@ -312,7 +315,7 @@ static void set_mixer_element(snd_mixer_t *mixer,const char *name, int level,Mix
 			switch(action){
 				case CAPTURE:
 				if (snd_mixer_selem_has_capture_volume(elem)){
-					snd_mixer_selem_get_playback_volume_range(elem, &sndMixerPMin, &sndMixerPMax);
+					snd_mixer_selem_get_capture_volume_range(elem, &sndMixerPMin, &sndMixerPMax);
 					newvol=(((sndMixerPMax-sndMixerPMin)*level)/100)+sndMixerPMin;
 					snd_mixer_selem_set_capture_volume_all(elem,newvol);
 					//g_message("Succesfully set capture level for %s.",elemname);
@@ -668,7 +671,9 @@ void alsa_write_process(MSFilter *obj){
 		while((size=im->b_wptr-im->b_rptr)>0){
 			samples=size/(2*ad->nchannels);
 			err=alsa_write(ad->handle,im->b_rptr,samples);
-			if (err>0) im->b_rptr+=err*(2*ad->nchannels);
+			if (err>0) {
+				im->b_rptr+=err*(2*ad->nchannels);
+			}
 			else break;
 		}
 		freemsg(im);
