@@ -84,17 +84,12 @@ static void on_timestamp_jump(RtpSession *s,uint32_t* ts, void * user_data)
 
 #endif
 
-static const char *ip4local="0.0.0.0";
 
-#ifdef INET6
-static const char *ip6local="::";
-#endif
-
-const char *get_local_addr_for(const char *remote)
-{
-	const char *ret;
+bool_t ms_is_ipv6(const char *remote){
+	bool_t ret=FALSE;
 #ifdef INET6
 	struct addrinfo hints, *res0;
+	
 	int err;
 	memset(&hints, 0, sizeof(hints));
 	hints.ai_family = PF_UNSPEC;
@@ -102,30 +97,24 @@ const char *get_local_addr_for(const char *remote)
 	err = getaddrinfo(remote,"8000", &hints, &res0);
 	if (err!=0) {
 		ms_warning ("get_local_addr_for: %s", gai_strerror(err));
-		return ip4local;
+		return FALSE;
 	}
-	ret=(res0->ai_addr->sa_family==AF_INET6) ? ip6local : ip4local; 
+	ret=(res0->ai_addr->sa_family==AF_INET6); 
 	freeaddrinfo(res0);
-#else
-	ret=ip4local;
 #endif
 	return ret;
 }
 
-RtpSession * create_duplex_rtpsession(RtpProfile *profile, int locport,const char *remip,int remport, int payload,int jitt_comp){
+RtpSession * create_duplex_rtpsession( int locport, bool_t ipv6){
 	RtpSession *rtpr;
 	rtpr=rtp_session_new(RTP_SESSION_SENDRECV);
 	rtp_session_set_recv_buf_size(rtpr,MAX_RTP_SIZE);
-	rtp_session_set_profile(rtpr,profile);
-	rtp_session_set_local_addr(rtpr,get_local_addr_for(remip),locport);
-	if (remport>0) rtp_session_set_remote_addr(rtpr,remip,remport);
 	rtp_session_set_scheduling_mode(rtpr,0);
 	rtp_session_set_blocking_mode(rtpr,0);
-	rtp_session_set_payload_type(rtpr,payload);
-	rtp_session_set_jitter_compensation(rtpr,jitt_comp);
 	rtp_session_enable_adaptive_jitter_compensation(rtpr,TRUE);
 	rtp_session_set_symmetric_rtp(rtpr,TRUE);
-	/*rtp_session_signal_connect(rtpr,"timestamp_jump",(RtpCallback)on_timestamp_jump,NULL);*/
+	rtp_session_set_local_addr(rtpr,ipv6 ? "::" : "0.0.0.0",locport);
+
 	return rtpr;
 }
 
@@ -168,13 +157,16 @@ static void payload_type_changed(RtpSession *session, unsigned long data){
 }
 
 
-AudioStream * audio_stream_start_full(RtpProfile *profile, int locport, const char *remip,int remport, int payload,int jitt_comp, const char *infile, const char *outfile, MSSndCard *playcard, MSSndCard *captcard, bool_t use_ec)
+int audio_stream_start_full(AudioStream *stream, RtpProfile *profile, const char *remip,int remport, int payload,int jitt_comp, const char *infile, const char *outfile, MSSndCard *playcard, MSSndCard *captcard, bool_t use_ec)
 {
-	AudioStream *stream=ms_new0(AudioStream,1);
-	RtpSession *rtps;
+	RtpSession *rtps=stream->session;
 	PayloadType *pt;
-	
-	rtps=create_duplex_rtpsession(profile,locport,remip,remport,payload,jitt_comp);
+	int tmp;	
+
+	rtp_session_set_profile(rtps,profile);
+	if (remport>0) rtp_session_set_remote_addr(rtps,remip,remport);
+	rtp_session_set_payload_type(rtps,payload);
+	rtp_session_set_jitter_compensation(rtps,jitt_comp);
 	
 	stream->rtpsend=ms_filter_new(MS_RTP_SEND_ID);
 	if (remport>0)
@@ -203,15 +195,14 @@ AudioStream * audio_stream_start_full(RtpProfile *profile, int locport, const ch
 	pt=rtp_profile_get_payload(profile,payload);
 	if (pt==NULL){
 		ms_error("audiostream.c: undefined payload type.");
-		return NULL;
+		return -1;
 	}
 	stream->encoder=ms_filter_create_encoder(pt->mime_type);
 	stream->decoder=ms_filter_create_decoder(pt->mime_type);
 	if ((stream->encoder==NULL) || (stream->decoder==NULL)){
 		/* big problem: we have not a registered codec for this payload...*/
-		audio_stream_free(stream);
 		ms_error("mediastream.c: No decoder available for payload %i.",payload);
-		return NULL;
+		return -1;
 	}
 	
 	if (use_ec) {
@@ -222,12 +213,16 @@ AudioStream * audio_stream_start_full(RtpProfile *profile, int locport, const ch
 	/* give the sound filters some properties */
 	ms_filter_call_method(stream->soundread,MS_FILTER_SET_SAMPLE_RATE,&pt->clock_rate);
 	ms_filter_call_method(stream->soundwrite,MS_FILTER_SET_SAMPLE_RATE,&pt->clock_rate);
+	tmp=1;
+	ms_filter_call_method(stream->soundwrite,MS_FILTER_SET_NCHANNELS, &tmp);
 	
 	/* give the encoder/decoder some parameters*/
 	ms_filter_call_method(stream->encoder,MS_FILTER_SET_SAMPLE_RATE,&pt->clock_rate);
-	ms_filter_call_method(stream->encoder,MS_FILTER_SET_BITRATE,&pt->normal_bitrate);
+	if (pt->normal_bitrate>0){
+		ms_message("Setting audio encoder network bitrate to %i",pt->normal_bitrate);
+		ms_filter_call_method(stream->encoder,MS_FILTER_SET_BITRATE,&pt->normal_bitrate);
+	}
 	ms_filter_call_method(stream->decoder,MS_FILTER_SET_SAMPLE_RATE,&pt->clock_rate);
-	ms_filter_call_method(stream->decoder,MS_FILTER_SET_BITRATE,&pt->normal_bitrate);
 	
 	if (pt->send_fmtp!=NULL) ms_filter_call_method(stream->encoder,MS_FILTER_SET_FMTP, (void*)pt->send_fmtp);
 	if (pt->recv_fmtp!=NULL) ms_filter_call_method(stream->decoder,MS_FILTER_SET_FMTP,(void*)pt->recv_fmtp);
@@ -254,26 +249,31 @@ AudioStream * audio_stream_start_full(RtpProfile *profile, int locport, const ch
 	ms_ticker_attach(stream->ticker,stream->soundread);
 	ms_ticker_attach(stream->ticker,stream->rtprecv);
 	
-	return stream;
+	return 0;
 }
 
 
-AudioStream * audio_stream_start_with_files(RtpProfile *prof,int locport,const char *remip, int remport,int pt,int jitt_comp, const char *infile, const char * outfile)
+int audio_stream_start_with_files(AudioStream *stream, RtpProfile *prof,const char *remip, int remport,int pt,int jitt_comp, const char *infile, const char * outfile)
 {
-	return audio_stream_start_full(prof,locport,remip,remport,pt,jitt_comp,infile,outfile,NULL,NULL,FALSE);
+	return audio_stream_start_full(stream,prof,remip,remport,pt,jitt_comp,infile,outfile,NULL,NULL,FALSE);
 }
 
 AudioStream * audio_stream_start(RtpProfile *prof,int locport,const char *remip,int remport,int profile,int jitt_comp,bool_t use_ec)
 {
 	MSSndCard *sndcard;
+	AudioStream *stream;
 	sndcard=ms_snd_card_manager_get_default_card(ms_snd_card_manager_get());
 	if (sndcard==NULL)
 		return NULL;
-	return audio_stream_start_full(prof,locport,remip,remport,profile,jitt_comp,NULL,NULL,sndcard,sndcard,use_ec);
+	stream=audio_stream_new(locport, ms_is_ipv6(remip));
+	if (audio_stream_start_full(stream,prof,remip,remport,profile,jitt_comp,NULL,NULL,sndcard,sndcard,use_ec)==0) return stream;
+	audio_stream_free(stream);
+	return NULL;
 }
 
-AudioStream *audio_stream_start_with_sndcards(RtpProfile *prof,int locport,const char *remip,int remport,int profile,int jitt_comp,MSSndCard *playcard, MSSndCard *captcard, bool_t ec)
+AudioStream *audio_stream_start_with_sndcards(RtpProfile *prof,int locport,const char *remip,int remport,int profile,int jitt_comp,MSSndCard *playcard, MSSndCard *captcard, bool_t use_ec)
 {
+	AudioStream *stream;
 	if (playcard==NULL) {
 		ms_error("No playback card.");
 		return NULL;
@@ -282,7 +282,10 @@ AudioStream *audio_stream_start_with_sndcards(RtpProfile *prof,int locport,const
 		ms_error("No capture card.");
 		return NULL;
 	}
-	return audio_stream_start_full(prof,locport,remip,remport,profile,jitt_comp,NULL,NULL,playcard,captcard,ec);
+	stream=audio_stream_new(locport, ms_is_ipv6(remip));
+	if (audio_stream_start_full(stream,prof,remip,remport,profile,jitt_comp,NULL,NULL,playcard,captcard,use_ec)==0) return stream;
+	audio_stream_free(stream);
+	return NULL;
 }
 
 void audio_stream_set_rtcp_information(AudioStream *st, const char *cname, const char *tool){
@@ -313,28 +316,38 @@ void audio_stream_record(AudioStream *st, const char *name){
 	}
 }
 
+
+AudioStream *audio_stream_new(int locport, bool_t ipv6){
+	AudioStream *stream=ms_new0(AudioStream,1);
+	stream->session=create_duplex_rtpsession(locport,ipv6);
+	return stream;
+}
+int audio_stream_start_now(AudioStream *stream, RtpProfile * prof,  const char *remip, int remport, int payload_type, int jitt_comp, MSSndCard *playcard, MSSndCard *captcard, bool_t use_ec){
+	return audio_stream_start_full(stream,prof,remip,remport,payload_type,jitt_comp,NULL,NULL,playcard,captcard,use_ec);
+}
+
 void audio_stream_stop(AudioStream * stream)
 {
-	
-	ms_ticker_detach(stream->ticker,stream->soundread);
-	ms_ticker_detach(stream->ticker,stream->rtprecv);
-	
-	rtp_stats_display(rtp_session_get_stats(stream->session),"Audio session's RTP statistics");
-
-	if (stream->ec!=NULL){
-		ms_filter_unlink(stream->soundread,0,stream->ec,1);
-		ms_filter_unlink(stream->ec,1,stream->encoder,0);
-		ms_filter_unlink(stream->dtmfgen,0,stream->ec,0);
-		ms_filter_unlink(stream->ec,0,stream->soundwrite,0);
-	}else{
-		ms_filter_unlink(stream->soundread,0,stream->encoder,0);
-		ms_filter_unlink(stream->dtmfgen,0,stream->soundwrite,0);
+	if (stream->ticker){
+		ms_ticker_detach(stream->ticker,stream->soundread);
+		ms_ticker_detach(stream->ticker,stream->rtprecv);
+		
+		rtp_stats_display(rtp_session_get_stats(stream->session),"Audio session's RTP statistics");
+		
+		if (stream->ec!=NULL){
+			ms_filter_unlink(stream->soundread,0,stream->ec,1);
+			ms_filter_unlink(stream->ec,1,stream->encoder,0);
+			ms_filter_unlink(stream->dtmfgen,0,stream->ec,0);
+			ms_filter_unlink(stream->ec,0,stream->soundwrite,0);
+		}else{
+			ms_filter_unlink(stream->soundread,0,stream->encoder,0);
+			ms_filter_unlink(stream->dtmfgen,0,stream->soundwrite,0);
+		}
+		
+		ms_filter_unlink(stream->encoder,0,stream->rtpsend,0);
+		ms_filter_unlink(stream->rtprecv,0,stream->decoder,0);
+		ms_filter_unlink(stream->decoder,0,stream->dtmfgen,0);
 	}
-
-	ms_filter_unlink(stream->encoder,0,stream->rtpsend,0);
-	ms_filter_unlink(stream->rtprecv,0,stream->decoder,0);
-	ms_filter_unlink(stream->decoder,0,stream->dtmfgen,0);
-	
 	audio_stream_free(stream);
 }
 
@@ -348,7 +361,11 @@ RingStream * ring_start_with_cb(const char *file,int interval,MSSndCard *sndcard
 	int tmp;
 	stream=ms_new0(RingStream,1);
 	stream->source=ms_filter_new(MS_FILE_PLAYER_ID);
-	ms_filter_call_method(stream->source,MS_FILE_PLAYER_OPEN,(void*)file);
+	if (ms_filter_call_method(stream->source,MS_FILE_PLAYER_OPEN,(void*)file)<0){
+		ms_filter_destroy(stream->source);
+		ms_free(stream);
+		return NULL;
+	}
 	ms_filter_call_method(stream->source,MS_FILE_PLAYER_LOOP,&interval);
 	ms_filter_call_method_noarg(stream->source,MS_FILE_PLAYER_START);
 	if (func!=NULL)
