@@ -317,6 +317,19 @@ static int receiver_set_sdpcandidates(MSFilter * f, void *arg)
 	return 0;
 }
 
+static void receiver_preprocess(MSFilter * f){
+	ReceiverData *d = (ReceiverData *) f->data;
+	if (d->session){
+		PayloadType *pt=rtp_profile_get_payload(
+			rtp_session_get_profile(d->session),
+			rtp_session_get_recv_payload_type(d->session));
+		if (pt){
+			if (pt->type!=PAYLOAD_VIDEO)
+				rtp_session_flush_sockets(d->session);
+		}
+	}
+}
+
 static void receiver_process(MSFilter * f)
 {
 	ReceiverData *d = (ReceiverData *) f->data;
@@ -325,11 +338,8 @@ static void receiver_process(MSFilter * f)
 
 	if (d->session == NULL)
 		return;
-#if !defined(_WIN32_WCE)
-	timestamp = (f->ticker->time * d->rate) / 1000LL;
-#else
+
 	timestamp = (f->ticker->time * d->rate) / ((uint64_t)1000);
-#endif
 	while ((m = rtp_session_recvm_with_ts(d->session, timestamp)) != NULL) {
 		mblk_t *payload = m->b_cont;
 
@@ -378,7 +388,7 @@ MSFilterDesc ms_rtp_recv_desc = {
 	0,
 	1,
 	receiver_init,
-	NULL,
+	receiver_process,
 	receiver_process,
 	receiver_postprocess,
 	receiver_uninit,
@@ -395,6 +405,7 @@ MSFilterDesc ms_rtp_recv_desc = {
 	.ninputs = 0,
 	.noutputs = 1,
 	.init = receiver_init,
+	.preprocess = receiver_preprocess,
 	.process = receiver_process,
 	.postprocess=receiver_postprocess,
 	.uninit = receiver_uninit,
