@@ -33,7 +33,6 @@ typedef struct EncState{
 	int vbr;
 	int cng;
 	int mode;
-	int penh;
 	int frame_size;
 	void *state;
 	uint32_t ts;
@@ -49,7 +48,6 @@ static void enc_init(MSFilter *f){
 	s->mode=0;
 	s->vbr=1;
 	s->cng=0;
-	s->penh=0;
 	s->frame_size=0;
 	s->state=0;
 	s->ts=0;
@@ -140,55 +138,62 @@ static void enc_preprocess(MSFilter *f){
 	}
 
 	if (s->rate==8000){
-		if (s->mode<=0 || s->mode>=9)
-			s->mode = 3; /* default mode */
-
-		if (s->mode==1)
-			s->bitrate = 2150;
-		else if (s->mode==2)
-			s->bitrate = 5950;
-		else if (s->mode==3)
-			s->bitrate = 8000;
-		else if (s->mode==4)
-			s->bitrate = 11000;
-		else if (s->mode==5)
-			s->bitrate = 15000;
-		else if (s->mode==6)
-			s->bitrate = 18200;
-		else if (s->mode==7)
-			s->bitrate = 24600;
-		else if (s->mode==8)
-			s->bitrate = 3950;
-
-		if (s->bitrate!=-1){
-			if (speex_encoder_ctl(s->state,SPEEX_SET_BITRATE,&s->bitrate)!=0){
-				ms_error("Could not set bitrate %i to speex encoder.",s->bitrate);
+		if (s->mode!=0){/* mode is set*/
+			if (s->mode<=0 || s->mode>=9)
+				s->mode = 3; /* default mode */
+	
+			if (s->mode==1)
+				s->bitrate = 2150;
+			else if (s->mode==2)
+				s->bitrate = 5950;
+			else if (s->mode==3)
+				s->bitrate = 8000;
+			else if (s->mode==4)
+				s->bitrate = 11000;
+			else if (s->mode==5)
+				s->bitrate = 15000;
+			else if (s->mode==6)
+				s->bitrate = 18200;
+			else if (s->mode==7)
+				s->bitrate = 24600;
+			else if (s->mode==8)
+				s->bitrate = 3950;
+	
+			if (s->bitrate!=-1){
+				if (speex_encoder_ctl(s->state,SPEEX_SET_BITRATE,&s->bitrate)!=0){
+					ms_error("Could not set bitrate %i to speex encoder.",s->bitrate);
+				}
 			}
 		}
 	}
 	else
 	{
-		int q=0;
-		s->bitrate = -1; /* 28000; */
-		if (s->mode<=0 || s->mode>=8)
-			s->mode = 6; /* default mode */
-		/* no table exist for wide and ultra:
-		From libspeex/mode.c, those values seems to make sense... */
-		if (s->mode<=5)
-			q=5;
-		else if (s->mode==5)
-			q=6;
-		else if (s->mode==6)
-			q=8;
-		else if (s->mode>=7)
-			q=10;
+		if (s->mode!=0){
+			int q=0;
+			s->bitrate = -1; /* 28000; */
+			if (s->mode<=0 || s->mode>=8)
+				s->mode = 6; /* default mode */
+			/* no table exist for wide and ultra:
+			From libspeex/mode.c, those values seems to make sense... */
+			if (s->mode<=5)
+				q=5;
+			else if (s->mode==5)
+				q=6;
+			else if (s->mode==6)
+				q=8;
+			else if (s->mode>=7)
+				q=10;
 
-		if (speex_encoder_ctl(s->state,SPEEX_SET_QUALITY,&q)!=0){
-			ms_error("Could not set quality %i to speex encoder.",q);
+			if (speex_encoder_ctl(s->state,SPEEX_SET_QUALITY,&q)!=0){
+				ms_error("Could not set quality %i to speex encoder.",q);
+			}
 		}
 	}
 	if (s->maxbitrate>0){
-		if (speex_encoder_ctl(s->state,SPEEX_SET_BITRATE,&s->maxbitrate)!=0){
+		/* convert from network bitrate to codec bitrate:*/
+		/* ((nbr/(50*8)) -20-12-8)*50*8*/
+		int cbr=(((float)s->maxbitrate/(50.0*8))-20-12-8)*50*8;
+		if (speex_encoder_ctl(s->state,SPEEX_SET_BITRATE,&cbr)!=0){
 			ms_error("Could not set maxbitrate %i to speex encoder.",s->bitrate);
 		}
 	}
@@ -196,8 +201,6 @@ static void enc_preprocess(MSFilter *f){
 			ms_error("Could not get bitrate %i to speex encoder.",s->bitrate);
 	}
 	else ms_message("Using bitrate %i for speex encoder.",s->bitrate);
-	if (s->penh==1)
-		speex_decoder_ctl (s->state, SPEEX_SET_ENH, &s->penh);
 
 	speex_mode_query(mode,SPEEX_MODE_FRAME_SIZE,&s->frame_size);
 }
@@ -244,6 +247,7 @@ static void enc_process(MSFilter *f){
 
 		mblk_set_timestamp_info(om,s->ts-s->frame_size);
 		ms_queue_put(f->outputs[0],om);
+		speex_bits_destroy(&bits);
 	}
 }
 
@@ -361,8 +365,8 @@ static int enc_add_attr(MSFilter *f, void *arg){
 static MSFilterMethod enc_methods[]={
 	{	MS_FILTER_SET_SAMPLE_RATE	,	enc_set_sr	},
 	{	MS_FILTER_SET_BITRATE		,	enc_set_br	},
-	{	MS_FILTER_ADD_FMTP			,	enc_add_fmtp },
-	{	MS_FILTER_ADD_ATTR			,	enc_add_attr},
+	{	MS_FILTER_ADD_FMTP		,	enc_add_fmtp },
+	{	MS_FILTER_ADD_ATTR		,	enc_add_attr},
 	{	0				,	NULL		}
 };
 
@@ -406,6 +410,7 @@ MSFilterDesc ms_speex_enc_desc={
 
 typedef struct DecState{
 	int rate;
+	int penh;
 	int frsz;
 	void *state;
 } DecState;
@@ -415,6 +420,7 @@ static void dec_init(MSFilter *f){
 	s->rate=8000;
 	s->frsz=0;
 	s->state=NULL;
+	s->penh=1;
 	f->data=s;
 }
 
@@ -442,6 +448,8 @@ static void dec_preprocess(MSFilter *f){
 	}
 	s->state=speex_decoder_init(mode);
 	speex_mode_query(mode,SPEEX_MODE_FRAME_SIZE,&s->frsz);
+	if (s->penh==1)
+		speex_decoder_ctl (s->state, SPEEX_SET_ENH, &s->penh);
 }
 
 static void dec_postprocess(MSFilter *f){
