@@ -30,12 +30,6 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 #include <malloc.h> /* for alloca */
 #endif
 
-#ifdef TEST_AEC
-#include <sys/types.h>
-#include <sys/stat.h>
-#include <fcntl.h>
-#endif
-
 static const int framesize=128;
 static const int filter_length=2048; /*250 ms*/
 
@@ -43,40 +37,27 @@ typedef struct SpeexECState{
 	SpeexEchoState *ecstate;
 	MSBufferizer in[2];
 	int framesize;
+	int filterlength;
+	int samplerate;
 	SpeexPreprocessState *den;
         int ref;
         int echo;
         int out;
 }SpeexECState;
 
-#if 0
-struct adataptedstate {
-   int frame_size;           /**< Number of samples processed each time */
-   int window_size;
-   int M;
-   int cancel_count;
-   int adapted;
-};
-#endif
-
 static void speex_ec_init(MSFilter *f){
 	SpeexECState *s=ms_new(SpeexECState,1);
+
+	s->samplerate=8000;
 	s->framesize=framesize;
+	s->filterlength=filter_length;
+
 	ms_bufferizer_init(&s->in[0]);
 	ms_bufferizer_init(&s->in[1]);
-	s->ecstate=speex_echo_state_init(s->framesize,filter_length);
+	s->ecstate=speex_echo_state_init(s->framesize,s->filterlength);
+	s->den = speex_preprocess_state_init(s->framesize, s->samplerate);
+
 	f->data=s;
-
-	s->den = speex_preprocess_state_init(s->framesize, 8000);
-#ifdef TEST_AEC
-	s->ref=-1;
-	s->echo=-1;
-	s->out=-1;
-	s->ref=open("aec_ref.wav",O_WRONLY|O_CREAT|O_TRUNC, S_IRUSR|S_IWUSR);
-	s->echo=open("aec_echo.wav",O_WRONLY|O_CREAT|O_TRUNC, S_IRUSR|S_IWUSR);
-	s->out=open("aec_out.wav",O_WRONLY|O_CREAT|O_TRUNC, S_IRUSR|S_IWUSR);
-#endif
-
 }
 
 static void speex_ec_uninit(MSFilter *f){
@@ -87,14 +68,6 @@ static void speex_ec_uninit(MSFilter *f){
 	if (s->den!=NULL)
 	  speex_preprocess_state_destroy(s->den);
 
-#ifdef TEST_AEC
-	if (s->ref!=-1)
-	  close(s->ref);
-	if (s->echo!=-1)
-	  close(s->echo);
-	if (s->out!=-1)
-	  close(s->out);
-#endif
 	ms_free(s);
 }
 
@@ -113,6 +86,9 @@ static void speex_ec_process(MSFilter *f){
 #else
 	float *noise=NULL;
 #endif
+#ifdef AMD_WIN32_HACK
+	static int count=0;
+#endif
 
 	/*read input and put in bufferizers*/
 	ms_bufferizer_put_from_queue(&s->in[0],f->inputs[0]);
@@ -122,17 +98,6 @@ static void speex_ec_process(MSFilter *f){
 
 	ms_debug("speexec:  in0=%i, in1=%i",ms_bufferizer_get_avail(&s->in[0]),ms_bufferizer_get_avail(&s->in[1]));
 
-	if (ms_bufferizer_get_avail(&s->in[0])> 320 * 6) /* above 250ms -> useless */
-	  {
-	    /* reset evrything */
-	    ms_warning("speexec: -reset of echo canceller- in0=%i, in1=%i",ms_bufferizer_get_avail(&s->in[0]),ms_bufferizer_get_avail(&s->in[1]));
-	    flushq(&s->in[1].q,0);
-	    flushq(&s->in[0].q,0);
-	    ms_bufferizer_init(&s->in[0]);
-	    ms_bufferizer_init(&s->in[1]);
-            speex_echo_state_reset(s->ecstate);
-	  }
-
 	while (ms_bufferizer_get_avail(&s->in[0])>=nbytes && ms_bufferizer_get_avail(&s->in[1])>=nbytes){
 		om0=allocb(nbytes,0);
 		ms_bufferizer_read(&s->in[0],(uint8_t*)om0->b_wptr,nbytes);
@@ -148,33 +113,32 @@ static void speex_ec_process(MSFilter *f){
 		speex_echo_cancel(s->ecstate,(short*)in1,(short*)om0->b_rptr,(short*)om1->b_wptr,noise);
 		if (s->den!=NULL && noise!=NULL)
 		  speex_preprocess(s->den, (short*)om1->b_wptr, noise);
-#ifdef TEST_AEC
-		if (s->ref!=-1 && s->echo!=-1 && s->out!=-1)
-		  {
-		    write(s->ref,in1,nbytes);
-		    write(s->echo,om0->b_rptr,nbytes);
-		    write(s->out,om1->b_wptr,nbytes);
-		  }
-#endif
-#if 0
-		{
-			static int adapted=0;
-			static int nonadapted=0;
-			struct adataptedstate *_adaptedstate=(struct adataptedstate *)s->ecstate;
-			if (_adaptedstate->adapted)
-				adapted++;
-			else
-				ms_message("nonadapted once");
-			if (adapted%100==1)
-				ms_message("adapted 100 times");
-		}
-#endif
 
 		om1->b_wptr+=nbytes;
 		ms_queue_put(f->outputs[1],om1);
-		
+#ifdef AMD_WIN32_HACK
+		count++;
+		if (count==100*3)
+		{
+			ms_message("periodic reset of echo canceller.");
+			speex_echo_state_reset(s->ecstate);
+			count=0;
+		}		
+#endif
 	}
-	while (ms_bufferizer_get_avail(&s->in[1])>2*nbytes){
+
+	if (ms_bufferizer_get_avail(&s->in[0])> 4*320*(s->samplerate/8000)) /* above 4*20ms -> useless */
+	  {
+	    /* reset evrything */
+	    ms_warning("speexec: -reset of echo canceller- in0=%i, in1=%i",ms_bufferizer_get_avail(&s->in[0]),ms_bufferizer_get_avail(&s->in[1]));
+	    flushq(&s->in[1].q,0);
+	    flushq(&s->in[0].q,0);
+	    ms_bufferizer_init(&s->in[0]);
+	    ms_bufferizer_init(&s->in[1]);
+	    speex_echo_state_reset(s->ecstate);
+	  }
+
+	while (ms_bufferizer_get_avail(&s->in[1])> 4*320*(s->samplerate/8000)){
 		om1=allocb(nbytes,0);
 		ms_bufferizer_read(&s->in[1],(uint8_t*)om1->b_wptr,nbytes);
 		om1->b_wptr+=nbytes;
@@ -188,16 +152,62 @@ static void speex_ec_process(MSFilter *f){
 static int speex_ec_set_sr(MSFilter *f, void *arg){
 #ifdef SPEEX_ECHO_SET_SAMPLING_RATE
 	SpeexECState *s=(SpeexECState*)f->data;
-	spx_int32_t tmp = *(int*)arg;
-	speex_echo_ctl(s->ecstate, SPEEX_ECHO_SET_SAMPLING_RATE, &tmp);
+
+	s->samplerate = *(int*)arg;
+
+	if (s->ecstate==NULL)
+		speex_echo_state_destroy(s->ecstate);
+	if (s->den!=NULL)
+	  speex_preprocess_state_destroy(s->den);
+
+	s->ecstate=speex_echo_state_init(s->framesize,s->filterlength);
+	speex_echo_ctl(s->ecstate, SPEEX_ECHO_SET_SAMPLING_RATE, &s->samplerate);
+	s->den = speex_preprocess_state_init(s->framesize, s->samplerate);
 #else
 	ms_error("Speex echocanceler does not support 16Khz sampling rate in this version!");
 #endif
 	return 0;
 }
 
+static int speex_ec_set_framesize(MSFilter *f, void *arg){
+	SpeexECState *s=(SpeexECState*)f->data;
+	s->framesize = *(int*)arg;
+
+	if (s->ecstate==NULL)
+		speex_echo_state_destroy(s->ecstate);
+	if (s->den!=NULL)
+	  speex_preprocess_state_destroy(s->den);
+
+	s->ecstate=speex_echo_state_init(s->framesize,s->filterlength);
+#ifdef SPEEX_ECHO_SET_SAMPLING_RATE
+	speex_echo_ctl(s->ecstate, SPEEX_ECHO_SET_SAMPLING_RATE, &s->samplerate);
+#endif
+	s->den = speex_preprocess_state_init(s->framesize, s->samplerate);
+	return 0;
+}
+
+static int speex_ec_set_filterlength(MSFilter *f, void *arg){
+	SpeexECState *s=(SpeexECState*)f->data;
+	s->filterlength = *(int*)arg;
+
+	if (s->ecstate==NULL)
+		speex_echo_state_destroy(s->ecstate);
+	if (s->den!=NULL)
+	  speex_preprocess_state_destroy(s->den);
+
+	s->ecstate=speex_echo_state_init(s->framesize,s->filterlength);
+#ifdef SPEEX_ECHO_SET_SAMPLING_RATE
+	speex_echo_ctl(s->ecstate, SPEEX_ECHO_SET_SAMPLING_RATE, &s->samplerate);
+#endif
+	s->den = speex_preprocess_state_init(s->framesize, s->samplerate);
+
+	return 0;
+}
+
 static MSFilterMethod speex_ec_methods[]={
 	{	MS_FILTER_SET_SAMPLE_RATE, speex_ec_set_sr },
+	{	MS_FILTER_SET_FRAMESIZE, speex_ec_set_framesize },
+	{	MS_FILTER_SET_FILTERLENGTH, speex_ec_set_filterlength },
 	{	0			, NULL}
 };
 
