@@ -28,9 +28,9 @@ typedef struct EncState{
 	theora_info tinfo;
 	yuv_buffer yuv;
 	mblk_t *packed_conf;
-	uint start_time;
-	uint conf_time;
-	uint mtu;
+	uint64_t start_time;
+	uint64_t conf_time;
+	unsigned int mtu;
 } EncState;
 
 static void enc_init(MSFilter *f){
@@ -42,7 +42,7 @@ static void enc_init(MSFilter *f){
 	s->tinfo.frame_height=MS_VIDEO_SIZE_CIF_H;
 	s->tinfo.offset_x=0;
 	s->tinfo.offset_y=0;
-	s->tinfo.target_bitrate=50000;
+	s->tinfo.target_bitrate=500000;
 	s->tinfo.pixelformat=OC_PF_420;
 	s->tinfo.fps_numerator=15;
 	s->tinfo.fps_denominator=1;
@@ -100,7 +100,7 @@ static int enc_set_fps(MSFilter *f, void *data){
 	EncState *s=(EncState*)f->data;
 	s->tinfo.fps_numerator=*fps;
 	s->tinfo.keyframe_frequency=(*fps)*5;
-	s->tinfo.keyframe_frequency_force=(*fps)*20;
+	s->tinfo.keyframe_frequency_force=(*fps)*5;
 	return 0;
 }
 
@@ -116,21 +116,33 @@ static int enc_set_br(MSFilter *f, void*data){
 	EncState *s=(EncState*)f->data;
 	MSVideoSize vsize;
 	float fps;
+	float codecbr=(float)br;
 	vsize.width=s->tinfo.width;
 	vsize.height=s->tinfo.height;
 	fps=s->tinfo.fps_numerator;
-	s->tinfo.target_bitrate=(float)br/1.2;
-	s->tinfo.keyframe_data_target_bitrate=br;
-	/*those default settings would need to affined*/
-	if (br>=256000){
-		vsize=MS_VIDEO_SIZE_CIF;
+	s->tinfo.target_bitrate=codecbr*0.8;
+	s->tinfo.keyframe_data_target_bitrate=codecbr;
+	/*those default settings would need to be affined*/
+	if (br>=512000){
+		vsize.width = MS_VIDEO_SIZE_CIF_W;
+		vsize.height = MS_VIDEO_SIZE_CIF_H;
+		s->tinfo.quality=32;
 		fps=15;
-	}else if(br>=128000){
-		vsize=MS_VIDEO_SIZE_QCIF;
+	}else if (br>=256000){
+		vsize.width = MS_VIDEO_SIZE_CIF_W;
+		vsize.height = MS_VIDEO_SIZE_CIF_H;
+		s->tinfo.quality=5;
 		fps=12;
+	}else if(br>=128000){
+		vsize.width=MS_VIDEO_SIZE_QCIF_W;
+		vsize.height=MS_VIDEO_SIZE_QCIF_H;
+		s->tinfo.quality=20;
+		fps=10;
 	}else if(br>=64000){
-		vsize=MS_VIDEO_SIZE_QCIF;
-		fps=6;
+		vsize.width=MS_VIDEO_SIZE_QCIF_W;
+		vsize.height=MS_VIDEO_SIZE_QCIF_H;
+		s->tinfo.quality=7;
+		fps=7;
 	}
 	enc_set_vsize(f,&vsize);
 	enc_set_fps(f,&fps);
@@ -207,8 +219,9 @@ static int create_packed_conf(EncState *s){
 
 static void enc_preprocess(MSFilter *f){
 	EncState *s=(EncState*)f->data;
-	if (theora_encode_init(&s->tstate,&s->tinfo)!=0){
-		ms_error("error in theora_encode_init() !");
+	int err;
+	if ((err=theora_encode_init(&s->tstate,&s->tinfo))!=0){
+		ms_error("error in theora_encode_init() : %i !",err);
 	}
 	s->yuv.y_width=s->tinfo.width;
 	s->yuv.y_height=s->tinfo.height;
@@ -275,23 +288,41 @@ static void packetize_and_send(MSFilter *f, EncState *s, mblk_t *om, uint32_t ti
 	}
 }
 
+bool_t need_send_conf(EncState *s, uint64_t elapsed){
+#ifdef AMD_HACK
+	if (elapsed<5000 && elapsed>=s->conf_time){
+		s->conf_time+=500;
+		return TRUE;
+	}
+#else
+	/*send immediately then 10 seconds later */
+	if ( (elapsed<1000 && s->conf_time==0)
+		|| (elapsed>10000 && s->conf_time==1)){
+		s->conf_time++;
+		return TRUE;
+	}
+#endif
+	return FALSE;
+}
+
 static void enc_process(MSFilter *f){
 	mblk_t *im,*om;
 	ogg_packet op;
 	EncState *s=(EncState*)f->data;
-	uint timems=f->ticker->time;
+	uint64_t timems=f->ticker->time;
 	uint32_t timestamp=timems*90;
-	int elapsed=timems-s->start_time;
+	uint64_t elapsed=timems-s->start_time;
 	while((im=ms_queue_get(f->inputs[0]))!=NULL){
 		/*for the firsts frames only send theora packed conf*/
 		om=NULL;
-		if (elapsed<1000){
-			/* during the first second, only send packed conf twice */
-			if (elapsed>=s->conf_time){
-				s->conf_time+=500;
+
+		if (need_send_conf(s,elapsed)){
+			if (s->packed_conf) {
 				om=dupmsg(s->packed_conf);
-				ms_debug("sending theora packed conf");
+				ms_message("sending theora packed conf (%i bytes)",msgdsize(om));
 				packetize_and_send(f,s,om,timestamp,THEORA_PACKED_CONF);
+			}else {
+				ms_error("No packed conf to send.");
 			}
 		}else{
 			enc_fill_yuv(&s->yuv,im);
@@ -323,6 +354,26 @@ static MSFilterMethod enc_methods[]={
 	{	0			, NULL }
 };
 
+#ifdef _MSC_VER
+
+MSFilterDesc ms_theora_enc_desc={
+	MS_THEORA_ENC_ID,
+	"MSTheoraEnc",
+	"The theora video encoder from xiph.org",
+	MS_FILTER_ENCODER,
+	"theora",
+	1,
+	1,
+	enc_init,
+	enc_preprocess,
+	enc_process,
+	enc_postprocess,
+	enc_uninit,
+	enc_methods
+};
+
+#else
+
 MSFilterDesc ms_theora_enc_desc={
 	.id=MS_THEORA_ENC_ID,
 	.name="MSTheoraEnc",
@@ -339,6 +390,8 @@ MSFilterDesc ms_theora_enc_desc={
 	.methods=enc_methods
 };
 
+#endif
+
 MS_FILTER_DESC_EXPORT(ms_theora_enc_desc)
 
 typedef struct DecState{
@@ -368,10 +421,10 @@ static void dec_uninit(MSFilter *f){
 
 static bool_t dec_init_theora(DecState *s, ogg_packet *op){
 	theora_comment tcom;
+	static const int ident_packet_size=42; 
 	theora_comment_init(&tcom);
 	tcom.vendor="dummy";
 	op->b_o_s=1;
-	static const int ident_packet_size=42; 
 	if (theora_decode_header(&s->tinfo,&tcom,op)==0){
 		op->packet+=ident_packet_size;
 		op->bytes-=ident_packet_size;
@@ -487,6 +540,26 @@ static void dec_process(MSFilter *f){
 	}
 }
 
+#ifdef _MSC_VER
+
+MSFilterDesc ms_theora_dec_desc={
+	MS_THEORA_DEC_ID,
+	"MSTheoraDec",
+	"The theora video decoder from xiph.org",
+	MS_FILTER_DECODER,
+	"theora",
+	1,
+	1,
+	dec_init,
+	NULL,
+	dec_process,
+	NULL,
+	dec_uninit,
+	NULL
+};
+
+#else
+
 MSFilterDesc ms_theora_dec_desc={
 	.id=MS_THEORA_DEC_ID,
 	.name="MSTheoraDec",
@@ -500,4 +573,5 @@ MSFilterDesc ms_theora_dec_desc={
 	.uninit=dec_uninit
 };
 
+#endif
 MS_FILTER_DESC_EXPORT(ms_theora_dec_desc)
