@@ -17,11 +17,12 @@ along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
 
+#include <ffmpeg/avcodec.h>
+
 #include "mediastreamer2/msfilter.h"
 #include "mediastreamer2/msvideo.h"
 #include "rfc2429.h"
 
-#include <ffmpeg/avcodec.h>
 
 extern void ms_ffmpeg_check_init();
 
@@ -33,6 +34,8 @@ typedef struct DecState{
 	mblk_t *yuv_msg;
 	int output_pix_fmt;
 	MSVideoSize vsize;
+	uint8_t dci[512];
+	int dci_size;
 }DecState;
 
 
@@ -44,7 +47,7 @@ static int dec_set_vsize(MSFilter *f, void *arg){
 }
 
 static void dec_init(MSFilter *f, enum CodecID cid){
-	DecState *s=ms_new(DecState,1);
+	DecState *s=ms_new0(DecState,1);
 	ms_ffmpeg_check_init();
 	
 	avcodec_get_context_defaults(&s->av_context);
@@ -78,6 +81,26 @@ static void dec_uninit(MSFilter *f){
 	ms_free(s);
 }
 
+static int dec_add_fmtp(MSFilter *f, void *data){
+	const char *fmtp=(const char*)data;
+	DecState *s=(DecState*)f->data;
+	char config[512];
+	if (fmtp_get_value(fmtp,"config",config,sizeof(config))){
+		/*convert hexa decimal config string into a bitstream */
+		int i,j,max=strlen(config);
+		char octet[3];
+		octet[2]=0;
+		for(i=0,j=0;i<max;i+=2,++j){
+			octet[0]=config[i];
+			octet[1]=config[i+1];
+			s->dci[j]=strtol(octet,NULL,16);
+		}
+		s->dci_size=j;
+		ms_message("Got mpeg4 config string: %s",config);
+	}
+	return 0;
+}
+
 static void dec_preprocess(MSFilter *f){
 	DecState *s=(DecState*)f->data;
 	int error;
@@ -88,6 +111,10 @@ static void dec_preprocess(MSFilter *f){
 	}
 	error=avcodec_open(&s->av_context, s->av_codec);
 	if (error!=0) ms_error("avcodec_open() failed: %i",error);
+	if (s->codec==CODEC_ID_MPEG4 && s->dci_size>0){
+		s->av_context.extradata=s->dci;
+		s->av_context.extradata_size=s->dci_size;
+	}
 }
 
 static void dec_postprocess(MSFilter *f){
@@ -183,13 +210,14 @@ static void dec_process_frame(MSFilter *f, mblk_t *inm){
 			mblk_t *frame;
 			int remain,len;
 			/*ms_message("got marker bit !");*/
-			
-			msgpullup(s->input,-1);
+			/*append some padding bytes for ffmpeg to safely 
+			read extra bytes...*/
+			msgpullup(s->input,msgdsize(s->input)+8);
 			frame=s->input;
 			s->input=NULL;
 			while ( (remain=frame->b_wptr-frame->b_rptr)> 0) {
 				len=avcodec_decode_video(&s->av_context,&orig,&got_picture,(uint8_t*)frame->b_rptr,remain );
-				if (len<0) {
+				if (len<=0) {
 					ms_warning("ms_AVdecoder_process: error %i.",len);
 					break;
 				}
@@ -213,6 +241,7 @@ static void dec_process(MSFilter *f){
 
 static MSFilterMethod methods[]={
 	{		MS_FILTER_SET_VIDEO_SIZE	,	dec_set_vsize	},
+	{		MS_FILTER_ADD_FMTP		,	dec_add_fmtp	},
 	{		0		,		NULL			}
 };
 
